#include "construct.h"
#include "rankingtable.h"

#include <iostream.h>
#include <math.h>

Construct::Construct (float b, float cl, float prec)
{
	Mu = 0.0;
	Background = b;
	ConfLevel = cl;
        Precision = prec;
}

Construct::~Construct ()
{
}

void Construct::ShowVal (float min, float max, float step)
{
  int n1, n2;
  for (Mu = min; Mu <= max; Mu += step)
    {
      LimitN (&n1, &n2);
      cerr << "DLINE " << n1 << " " << n2 << " " << Mu << " " << Mu << endl;
    }
  return;
}

void Construct::LimitMu (int n, float *mu1, float *mu2)
{

  // this routine reverts the functions n1(mu)=n, n2(mu)=n
  // to give mu1 = mu(n1) and mu2= mu(n2)

  int n1, n2;

  // first search lower mu (= mu1)
	
  // first, find the value of mu for which n2 is bigger than n
  
  float StepWidth;
  if (n>0)		// adjust stepwidth, it will take ~sqrt(n) steps
    StepWidth  = float(n);
  else
    StepWidth = 1.0;	// make sure incrementation takes place at all

  bool found = false;   // check if we have found something...
  
  Mu = 0.0;
  
  while (!found)
    {
      LimitN (&n1, &n2);
      found = n2 > n;
//      cerr << n << " " << n2 << " " << Mu << endl;
      Mu += StepWidth;
    }
  
  Mu -= StepWidth; // we went one too far
  
  float LowerMu = 0.0;
  float CenterMu;
  float UpperMu = Mu;

  Mu = 0.0;
  // if n2 is bigger than n already at mu = 0, then we're already done...
  LimitN (&n1, &n2);

  found = n2 > n;	// was n2 > n
  
  while (!found)
    {
      CenterMu = 0.5 * (LowerMu + UpperMu);
//      cerr << LowerMu << " " << CenterMu << " " << UpperMu << endl;
      Mu = CenterMu;
      LimitN (&n1, &n2);
      
      if (n2 < n)
	LowerMu = CenterMu;
      else
	if (n2 > n)
	  UpperMu = CenterMu;
	else
	  {
	    found = true;
	  }
    }

  // now find the mu value which is the lowest possible
  // this is a bit unelegant...
  found = false;
  Mu = CenterMu;
  bool MuNegative = Mu < 0;
  while (!found && !MuNegative)
    {
      LimitN (&n1, &n2);
      found = n2 < n;
      Mu -= Precision;
      MuNegative = Mu < 0;
    }

  if (MuNegative)
    *mu1 = 0.0;
  else
    *mu1 = Mu + 2 * Precision;	
  

  // now the same business for finding the upper limit...
  Mu = *mu1;	// start at the lower value

  found = false;
  
  while (!found)
    {
      LimitN (&n1, &n2);
      found = n1 > n;
      Mu += StepWidth;
    }
  
  Mu -= StepWidth; // we went one too far
  
  LowerMu = *mu1;
  UpperMu = Mu;

  found = false;

  while (!found)
    {
      CenterMu = 0.5 * (LowerMu + UpperMu);
//      cerr << LowerMu << " " << CenterMu << " " << UpperMu << endl;
      Mu = CenterMu;
      LimitN (&n1, &n2);
      
      if (n1 < n)
	LowerMu = CenterMu;
      else
	if (n1 > n)
	  UpperMu = CenterMu;
	else
	  {
	    found = true;
	  }
    }

  // now find the highest possible mu value 
  // this is a bit unelegant...
  found = false;
  Mu = CenterMu;
  while (!found)
    {
      LimitN (&n1, &n2);
      found = n1 > n;
      Mu += Precision;
    }
  
  *mu2 = Mu - 2 * Precision;
  
  return;
  
}

void Construct::LimitN (int *n1, int *n2)
{

  // for a given Mu (stored in the privatevariable Mu), look which 
  // values of n are needed to include the desired confidence level
  // P(n in [n1, n2]|mu)=CL

  RankingTable Ranking;	// an object to manage the list needed to
			// perform the ordering

  // Alter the order of search:
  // start at n = mu + b (r=pmu/pmubest is maximal there)
  // and then proceed with n-1, n+1, n-2, n+2, n-3, ...
  // start with n-1 because shoulder of r(n) is steeper to the right.
  // *PN* 111200

  int n0 =  int (Mu + Background);
  				// r is maximal for n = mu + b; mind there
				// is conversion to integere here
  int n = n0;			// use n to calculate P
  bool decrease = true;		// governs change in n: increase if odd,
				// decrease if counts is even
  int i = 1;			// this is the value to be added/subtracted
				// to/from n

  float pmu;
  float mubest;
  float pmubest;
  float r;

  float AccProb = 0.0;	// probability accumulated by each value of n
  
  do
    {	
      pmu = P (n, Mu);
      AccProb += pmu;
      mubest = MuBest (n);
      pmubest = P (n, mubest);
      r = pmu / pmubest;
      // Insert into the Ranking table in descending order wrt r
      Ranking.EnterLine (n, pmu, mubest, pmubest, r);
      //	Ranking.Dump ();
      //	cerr << "==============================" << endl;
      if (decrease) // we decrease
	{
	  n = n0 - i;	// go to the 
	  decrease = false; // next time we have to increase
	}
	else // we increase
	{
	  n = n0 + i;
	  i++;	// we have completed one round, step to the next i;
	  decrease = true;
	}
    } while (AccProb < 0.99);
//      } while (AccProb <= ConfLevel);
  // .99 is necessary to find the global maximum of r(n)
  
  // this is the number of entries we need to reach the desired confidence
  // level:
  Ranking.GetN1N2 (n1, n2, ConfLevel);
//  Ranking.Dump ();
  
  return;

}

float Construct::P (int n, float mu)
{
  // calculates the Poisson probability P(n|mu+b)
  // use Stirling's approximation to avoid numerical trouble at high
  // values of n, mu
 if (n < 10) // normal n!
    return pow(mu+Background, n)
      * exp (-(mu+Background))
      / exp(gamma (n+1));	// gamma actually return ln (|Gamma|)
  else // above n = 10, the error between S(n) and n! is 0.8%
    return exp (n * log ((mu + Background) / n)
		- (mu + Background) 
		- 0.5 * log (n)
		+ n
		- 0.5 * log (2 * M_PI));
}

// give the mu for the highest P, gained from 0= dP/dmu
float Construct::MuBest (int n)
{
  float tmp = n - Background;

  if (tmp >= 0)
    return tmp;
  else
    return 0.;
}
