#include "globalmacros.h"
#include "options.h"
#include "construct.h"

#include <iostream.h>

/************************************************************************/
/*									*/
/* coli									*/
/* Peter Niessen, 100800						*/
/*									*/
/* Calculates confidence limits for Poisson process with a background   */
/* using the ordering principle proposed by Feldman/Cousins in		*/
/* A Unified Approach to the Classical Statistical Analysis of Small	*/
/* Signals, Phys. Rev. D, Vol. 57, No 7 (1998) 3873			*/
/*									*/
/* Usage: see output of coli -h						*/
/*									*/
/************************************************************************/
int main (int argc, char **argv)
{
	
  Options MyOptions (argc, argv);
  if (MyOptions.Status () != OK)
    return 0;
  // This is the object which does the construction of Conf Limit
  Construct MyConstruct (MyOptions.GetBackground (),
			 MyOptions.GetConfLevel (),
			 MyOptions.GetPrecision ());

  // check if we want debugging only
  if (MyOptions.Debug ())
    {
      MyConstruct.ShowVal (0., 2.0*MyOptions.GetBackground (), 0.01*MyOptions.Getn ());
//      MyConstruct.ShowVal (1.08, 1.15, 0.01);
      return -1;
    }

  int n = MyOptions.Getn ();
  float mu1, mu2;
  // Do the mu construction
  MyConstruct.LimitMu(n , &mu1, &mu2);
  // write to stdout
  cout << n << " " << mu1 << " " << mu2 << endl;
  
  return 0;

}
