#include "options.h"
#include "globalmacros.h"

#include <unistd.h>
#include <iostream.h>
#include <stdlib.h>


#define OPTSTRING "hn:b:c:p:d"

Options::Options (int argc, char **argv)
{
	int c;
	extern char *optarg;
	extern int optind, optopt;

	// some deafaults: 
	n = 0;	// the number of observed events
	Background = 0.;	// the expected background
	ConfLevel = 0.9;	// desired confidence level
	Precision = 0.01;	// numerical precision
	debug = false;		// debugging mode, make a kumac to draw lines

	while (EOF != ( c = getopt (argc, argv, OPTSTRING)))
	  {
	    switch (c)
	      {
	      case 'h': Usage ();
		status = OPT_NOP;
		return;
	      case 'n': n = atoi (optarg);
		break;
	      case 'b': Background = atof (optarg);
		break;
	      case 'c': ConfLevel = atof (optarg);
		break;
              case 'p': Precision = atof (optarg);
		break;
              case 'd': debug = true;
		break;
	      break;
		case ':': cerr << "Option -" << char(optopt);
			cerr << " requires an argument" << endl;
			status = OPT_NEEDSARG;
			return;
		case '?': cerr << "Unknown option -" << char(optopt) << endl;
			status = OPT_UNKNOWN;
			return;
		}
	}

	status = OK;
}


Options::~Options ()
{
}

int Options::Getn ()
{
	return n;
}

float Options::GetBackground ()
{
	return Background;
}

float Options::GetConfLevel ()
{
	return ConfLevel;
}

float Options::GetPrecision ()
{
	return Precision;
}

bool Options::Debug ()
{
	return debug;
}

int Options::Status ()
{
	return status;
}

void Options::Usage ()
{
  cerr << "coli gets you the Feldman-Cousins confidence limits" << endl;
  cerr << "Output is n mu1 mu2" << endl;
  cerr << "-n number of observed events,";
  cerr << " default: 0" << endl;
  cerr << "-b number of background, default: 0" << endl;
  cerr << "-c desired confidence level, default: 0.9" << endl;
  cerr << "-p precision, default 0.01" << endl;
  cerr << "-h: display this message" << endl;
  cerr << "Example: coli -b3 -n9 -p 0.1" << endl;
  return;
}
