#include "globalmacros.h"
#include "rankingtable.h"

#include <iostream.h>

RankingLineT::RankingLineT (int n_, float PMu_, float MuBest_,
			    float PMuBest_, float R_)
{
  n = n_;	// the n
  PMu = PMu_;   // P(n,mu)
  MuBest = MuBest_;	// the mu giving highest prob for given n
  PMuBest = PMuBest_;	// and the associated probability
  R = R_;		// Ratio Pmu/PMuBest
  bigger = NULL;	// tree like structure: here goes the bigger stuff 
  smaller = NULL;	// and her the smaller
}

RankingLineT::~RankingLineT ()
{
}

RankingTable::RankingTable ()
{
  Root = NULL;
  NElements = 0;
}

RankingTable::~RankingTable ()
{
  // delete the ranking table
  PrivDelete (Root);
  Root = NULL;
}

int RankingTable::EnterLine (int n, float PMu,
			     float MuBest, float PMuBest,
			     float R)
{

  // Enter an element into the table
  PrivEnterLine (n, PMu, MuBest, PMuBest, R, Root);
  ++NElements;

  return NElements;
}

void RankingTable::GetN1N2 (int *n1, int *n2, float ConfLevel)
{
  // find the n1 and n2 neede to include for the desired ConfLevel
  float AccProb = 0.0;
  bool FirstN = false;
  
  PrivGetN1N2 (&AccProb, &FirstN, n1, n2, ConfLevel, Root);
  
  return;
}

void RankingTable::Dump ()
{
  // Dump it on the screen
  PrivDump (Root);
  return;
}

void RankingTable::PrivEnterLine (int n, float PMu,
				  float MuBest, float PMuBest, float R,
				  RankingLineT *&RankingPtr)
{

  // Enter the element:
  // if there's nothing yet,
  if (NULL == RankingPtr)
    // create a new entry
    RankingPtr = new RankingLineT (n, PMu, MuBest, PMuBest, R);
  else
    // if it's smaller, put it into the branch which takes the smaller entries
    if (R < RankingPtr->R)
      PrivEnterLine (n, PMu, MuBest, PMuBest, R, RankingPtr->smaller);
    else 
      // put it into the branch which takes the bigger entries
      PrivEnterLine (n, PMu, MuBest, PMuBest, R, RankingPtr->bigger);

  return;

}

void RankingTable::PrivGetN1N2 (float *AccProb, bool *FirstN,
				int *n1, int *n2,
				float ConfLevel, RankingLineT *RankingPtr)
{
  if (NULL != RankingPtr)
    {
      PrivGetN1N2 (AccProb, FirstN, n1, n2, ConfLevel, RankingPtr->bigger);
      // At this place, we loop through R in descending order.
      if (*AccProb < ConfLevel)	// check if we have enough prob for now
	{
	  if (!*FirstN)  // initialise, one time only
	    {
	      *FirstN = true;
	      *n1 = RankingPtr->n;
	      *n2 = RankingPtr->n;
	    }	
	  *AccProb += RankingPtr->PMu;	// check the max and min values of n
	  if (RankingPtr->n < *n1)
	    *n1 = RankingPtr->n;
	  if (RankingPtr->n > *n2)
	    *n2 =RankingPtr->n;
	}
      else
	return;	// enough probability, return..
      PrivGetN1N2 (AccProb, FirstN, n1, n2, ConfLevel, RankingPtr->smaller);
    }
  
  return;
}

void RankingTable::PrivDelete (RankingLineT *&RankingPtr)
{
  if (NULL != RankingPtr)
    {
      PrivDelete (RankingPtr->smaller);
      PrivDelete (RankingPtr->bigger);
      delete RankingPtr;
      RankingPtr = NULL;
    }
  
  return;
}

void RankingTable::PrivDump (RankingLineT *RankingPtr)
{

  // diagnostics only
  if (NULL != RankingPtr)
    {
      
      PrivDump (RankingPtr->bigger);
      cerr << RankingPtr->n << " ";
      cerr << RankingPtr->PMu << " ";
      cerr << RankingPtr->MuBest << " ";
      cerr << RankingPtr->PMuBest << " ";
      cerr << RankingPtr->R << " ";
      cerr << endl;
      PrivDump (RankingPtr->smaller);
    }

  return;
}
